#include "avr/io.h"
#include "avr/interrupt.h"
#include "avr/wdt.h"

#define BRP_100_16	9
#define SJW_100_16	0
#define PRS_100_16	6
#define PHS2_100_16 3
#define PHS1_100_16 3
#define SMP_100_16 1
/* CANBT1=0x12;CANBT2=0x0C;CANBT3=0x37 */

#define DLC_MAX 8

void CAN_INIT(){
	CANGCON = 0x00;														/*DISABLE CAN*/
	while(CANGSTA&(1<<ENFG)); 											/*AUF DISABLE WARTEN*/
	for (uint8_t PAGE_INDEX=0;PAGE_INDEX<16;PAGE_INDEX++){ 				/* FOR ALL MOBS DO */
		CANPAGE = PAGE_INDEX << 4;										/*SELESCT MOB*/
		CANCDMOB=0x00; 													/*DISABLE MOB*/
		CANSTMOB=0x00;													/*CLEAR STATUS*/
		for (uint8_t MSG_INDEX=0;MSG_INDEX<DLC_MAX;MSG_INDEX++){CANMSG=0x00;}; /*CLEAR MESSAGE*/
	}
																		/*SET 100 kBit/s bei 16 MHz*/
	CANBT1 = (BRP_100_16  << 1);
	CANBT2 = (PRS_100_16  << 1) | (SJW_100_16  << 5); 
	CANBT3 = (PHS2_100_16 << 4) | (PHS1_100_16 << 1) | (SMP_100_16);
																		/*ENABLE INTERRUPTS davon Viele*/
	CANGIE = (1<<ENIT) | (1<<ENRX) | (1<<ENTX) | (1<<ENERR) | (1<<ENBX) | (1<<ENERG);
	CANIE1 = 0x7F; 
	CANIE2 = 0xFF;
																		/*Ein MOB zum Empfang einrichten*/
	CANPAGE = 0 << 4;													/*SELECT PAGE 0*/
	CANCDMOB = (0x02 << 6) | DLC_MAX;									/*enable reception & set DLC*/
	CANIDT1=0x00;CANIDT2=0x00;CANIDT3=0x00;CANIDT4=0x00;				/*IDs gelscht, Mask wir maximal*/
	CANIDM1=0x00;CANIDM2=0x00;CANIDM3=0x00;CANIDM4=0x00;				/*Maximale Maske fngt alles*/
	CANGCON = (1<<1);													/*ENABLE CAN*/
	while(!(CANGSTA&(1<<ENFG))); 										/*AUF ENABLE WARTEN*/
}

void LED_INIT(){
	DDRE  |= (1<<PE4); 
	PORTE |= (1<<PE4);
}

void LED_TOGGLE(){
	PORTE ^= (1<<PE4);}

ISR(CANIT_vect){
	LED_TOGGLE(); /*Etwas sichtbare machen*/
	/*MOB resetten*/
	CANPAGE = 0 << 4;													/*SELECT PAGE 0*/
	CANCDMOB = 0x00;													/*DISABLE MOB*/
	for (uint8_t MSG_INDEX=0;MSG_INDEX<DLC_MAX;MSG_INDEX++){CANMSG=0x00;}; /*CLEAR MESSAGE*/
	CANCDMOB = (0x02 << 6) | DLC_MAX;									/*enable reception & set DLC*/
	CANIDT1=0x00;CANIDT2=0x00;CANIDT3=0x00;CANIDT4=0x00;				/*IDs gelscht, Mask wir maximal*/
	CANIDM1=0x00;CANIDM2=0x00;CANIDM3=0x00;CANIDM4=0x00;				/*Maximale Maske fngt alles*/
	CANSTMOB = 0x00;
	CANCDMOB = (0x02 << 6) | DLC_MAX;									/*enable reception & set DLC*/
}

int main(){
	LED_INIT();  /* LED OFF */
	CAN_INIT();
	LED_TOGGLE();/* LED ON */
	sei();
	while(1){};
	return 0;
}
