//! @file $RCSfile: compiler.h,v $
//!
//! Copyright (c) 2005 Atmel.
//!
//! Use of this program is subject to Atmel's End User License Agreement.
//! Please read file license.txt for copyright notice.
//!
//! @brief This file redefines dedicated AVR COMPILER keywords in order
//! to ensure that any source file can be processed by these compilers.
//!
//! @version $Revision: 2.00 $ $Name: jtellier $
//!
//! @todo
//! @bug

#ifndef _COMPILER_H_
#define _COMPILER_H_

//_____ D E C L A R A T I O N S ____________________________________________
typedef unsigned char      Bool;

typedef unsigned char       U8 ;
typedef unsigned short      U16;
typedef unsigned long       U32;
typedef char                S8 ;
typedef short               S16;
typedef long                S32;

typedef union
{
  U32 w   ;     // w as WORD
  U16 h[2];     // h as HALF-WORD
  U8  b[4];     // b as BYTE
} Union32;

typedef union
{
  U16 h   ;     // h as HALF-WORD
  U8  b[2];     // b as BYTE
} Union16;

//_____ M A C R O S ________________________________________________________

// Some usefull macros...
#define Max(a, b)            ( (a)>(b) ? (a) : (b) )       // Take the max between a and b
#define Min(a, b)            ( (a)<(b) ? (a) : (b) )       // Take the min between a and b
#define Align_up(val, n)     ( ((val)+(n)-1) & ~((n)-1) )  // Around (up)   the number (val) on the (n) boundary
#define Align_down(val, n)   (  (val)        & ~((n)-1) )  // Around (down) the number (val) on the (n) boundary

// Bit and bytes manipulations
#define Low(U16)                ((U8)U16)
#define High(U16)               ((U8)(U16>>8))
#define Tst_bit_x(addrx,mask)   (*addrx & mask)
#define Set_bit_x(addrx,mask)   (*addrx = (*addrx |  mask))
#define Clr_bit_x(addrx,mask)   (*addrx = (*addrx & ~mask))

//! The default value to initialise pointers
#define  NULL              ((void *)0)

// Memory Type Location
#ifndef _CONST_TYPE_
#  define _CONST_TYPE_ code
#endif
#ifndef _MEM_TYPE_
#  define _MEM_TYPE_
#endif

// Constants
#define ENABLE   1
#define ENABLED  1
#define DISABLED 0
#define DISABLE  0
#define FALSE   (0==1)
#define TRUE    (1==1)
#define KO      0
#define OK      1
#define CLR     0
#define SET     1
#define OFF     0
#define ON      1

//__________________________________________________________________________

// IAR compiler for AVR products
#ifdef _IAR_AVR_

#  define __IOMACRO_H
#  define TID_GUARD(proc) ((__TID__ & 0x7FF0) != ((90 << 8) | ((proc) << 4)))

#  if !(__IAR_SYSTEMS_ICC__ > 2) && !defined(__IAR_SYSTEMS_ASM__)
#     error This file should only be compiled with IAR-iccavr or IAR-aavr.
#  endif // !(__IAR_SYSTEMS_ICC__ > 2) && !defined __IAR_SYSTEMS_ASM__

// The assembler uses a special set of macros...
#  ifdef __IAR_SYSTEMS_ASM__ // IAR-aavr

// Byte sized SFRs
#     define SFR_B_BITS(_NAME,_ADDR,_A,_B,_C,_D,_E,_F,_G,_H)\
         sfrb   _NAME = _ADDR
#     define SFR_B_BITS_EXT(_NAME,_ADDR,_A,_B,_C,_D,_E,_F,_G,_H)\
         sfrb   _NAME = _ADDR
#     define SFR_B2_BITS(_NAME1,_NAME2,_ADDR,_A,_B,_C,_D,_E,_F,_G,_H)\
         ASMSFRB2 _NAME1, _NAME2, _ADDR
ASMSFRB2 MACRO
   sfrb  \1 = \3
   sfrb  \2 = \3
   ENDM

// Word sized SFRs, needs to be expanded into an assembler macro first.
#     define SFR_W_BITS(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H, _I,_J,_K,_L,_M,_N,_O,_P)\
         ASMSFRW  _NAME, _ADDR
#     define SFR_W_BITS_EXT(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H, _I,_J,_K,_L,_M,_N,_O,_P)\
         ASMSFRW  _NAME, _ADDR
ASMSFRW  MACRO
   sfrw  \1  = \2
   sfrb  \1L = (\2+0)
   sfrb  \1H = (\2+1)
   ENDM

#  endif // __IAR_SYSTEMS_ASM__


#  ifdef __ICCAVR__ // IAR-iccavr
#     define __BYTEBITS(_NAME,_A,_B,_C,_D,_E,_F,_G,_H) \
      volatile U8                                      \
              _NAME ## _ ## _A:1,                      \
              _NAME ## _ ## _B:1,                      \
              _NAME ## _ ## _C:1,                      \
              _NAME ## _ ## _D:1,                      \
              _NAME ## _ ## _E:1,                      \
              _NAME ## _ ## _F:1,                      \
              _NAME ## _ ## _G:1,                      \
              _NAME ## _ ## _H:1;
#     define SFR_B_BITS(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H)           \
         __io union                                                      \
         {                                                               \
            volatile U8 _NAME;              /* The sfrb as 1 byte */     \
            struct                          /* The sfrb as 8 bits */     \
            {                                                            \
               __BYTEBITS(_NAME, _A,_B,_C,_D,_E,_F,_G,_H)                \
            };                                                           \
         } @ _ADDR;
#     define SFR_B2_BITS(_NAME1, _NAME2, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H) \
         __io union                                                      \
         {                                                               \
            volatile U8 _NAME1;             /* The sfrb as 1 byte */     \
            volatile U8 _NAME2;             /* The sfrb as 1 byte */     \
            struct                          /* The sfrb as 8 bits */     \
            {                                                            \
               __BYTEBITS(_NAME1, _A,_B,_C,_D,_E,_F,_G,_H)               \
            };                                                           \
            struct                          /* The sfrb as 8 bits */     \
            {                                                            \
               __BYTEBITS(_NAME2, _A,_B,_C,_D,_E,_F,_G,_H)               \
            };                                                           \
         } @ _ADDR;
#     define SFR_B_BITS_EXT(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H)      \
         __tiny __no_init volatile union                                \
         {                                                              \
            volatile U8 _NAME;              /* The sfrb as 1 byte */    \
            struct                          /* The sfrb as 8 bits */    \
            {                                                           \
               __BYTEBITS(_NAME, _A,_B,_C,_D,_E,_F,_G,_H)               \
            };                                                          \
         } @ _ADDR;
#     define SFR_W_BITS(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H, _I,_J,_K,_L,_M,_N,_O,_P)                           \
         __io union                                                                                               \
         {                                                                                                        \
            volatile U16 _NAME;                                                 /* The sfrw as 1 short       */   \
            struct                                                              /* The sfrw as 16 bits       */   \
            {                                                                                                     \
               __BYTEBITS(_NAME, _A,_B,_C,_D,_E,_F,_G,_H)                       /* Bit names defined by user */   \
               __BYTEBITS(_NAME, _I,_J,_K,_L,_M,_N,_O,_P)                       /* Bit names defined by user */   \
            };                                                                                                    \
            struct                                                              /* The sfrw as 2 bytes       */   \
            {                                                                                                     \
               volatile U8 _NAME ## L;                                                                            \
               volatile U8 _NAME ## H;                                                                            \
            };                                                                                                    \
            struct                                                              /* The sfrw as 2 x 8 bits      */ \
            {                                                                                                     \
               __BYTEBITS(_NAME ## L, Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)  /* Bit names hard coded to 0-7 */ \
               __BYTEBITS(_NAME ## H, Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)  /* Bit names hard coded to 0-7 */ \
            };                                                                                                    \
         } @ _ADDR;
#     define SFR_W_BITS_EXT(_NAME, _ADDR, _A,_B,_C,_D,_E,_F,_G,_H, _I,_J,_K,_L,_M,_N,_O,_P)                       \
         __tiny __no_init volatile union {                                                                        \
            volatile U16 _NAME;                                                                                   \
            struct                                                              /* The sfrw_ext as 16 bits   */   \
            {                                                                                                     \
               __BYTEBITS(_NAME, _A,_B,_C,_D,_E,_F,_G,_H)                       /* Bit names defined by user */   \
               __BYTEBITS(_NAME, _I,_J,_K,_L,_M,_N,_O,_P)                       /* Bit names defined by user */   \
            };                                                                                                    \
            struct   /* The sfrw _ext as 2 bytes */                                                               \
            {                                                                                                     \
               volatile U8 _NAME ## L;                                                                            \
               volatile U8 _NAME ## H;                                                                            \
            };                                                                                                    \
            struct                                                              /* The sfrw_ext as 2 x 8 bits  */ \
            {                                                                                                     \
               __BYTEBITS(_NAME ## L, Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)  /* Bit names hard coded to 0-7 */ \
               __BYTEBITS(_NAME ## H, Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)  /* Bit names hard coded to 0-7 */ \
            };                                                                                                    \
         } @ _ADDR;

#  endif // __ICCAVR__



#  define SFR_B(_NAME, _ADDR) SFR_B_BITS(_NAME, _ADDR,                              \
                                    Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)
#  define SFR_B2(_NAME1, _NAME2, _ADDR) SFR_B2_BITS(_NAME1, _NAME2, _ADDR,          \
                                    Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)
#  define SFR_B_EXT(_NAME, _ADDR) SFR_B_BITS_EXT(_NAME, _ADDR,                      \
                                    Bit0,Bit1,Bit2,Bit3,Bit4,Bit5,Bit6,Bit7)
#  define SFR_W(_NAME, _ADDR)  SFR_W_BITS(_NAME, _ADDR,                             \
                                    Bit0,Bit1,Bit2 ,Bit3 ,Bit4 ,Bit5 ,Bit6 ,Bit7 ,  \
                                    Bit8,Bit9,Bit10,Bit11,Bit12,Bit13,Bit14,Bit15)
#  define SFR_W_EXT(_NAME, _ADDR)  SFR_W_BITS_EXT(_NAME, _ADDR,                     \
                                    Bit0,Bit1,Bit2 ,Bit3 ,Bit4 ,Bit5 ,Bit6 ,Bit7 ,  \
                                    Bit8,Bit9,Bit10,Bit11,Bit12,Bit13,Bit14,Bit15)
#endif // _IAR_AVR_

#endif // _COMPILER_H_
